.include "ClkCfg.inc"
.include "DRAMCfg.inc"
.include "Addresses.inc"

NOINT      =  	0xc0

.text

.macro MOV_PC_LR
	mov pc,lr
.endm

.macro EQU
	=
.endm

.extern GPOS_Ticker
.extern _isr_Button

HAL_CLEAR_PENDING_BIT: .global HAL_CLEAR_PENDING_BIT
HAL_CLEAR_PENDING_BIT:
	SRCPND = 0x4a000000
	INTRPND = 0x4a000010	
	ldr	r1,=SRCPND
	str	r0,[r1]
	ldr	r1,=INTRPND
	str	r0,[r1]
	mov pc,lr
	
HAL_CLEAR_SUBPENDING_BIT: .global HAL_CLEAR_SUBPENDING_BIT
HAL_CLEAR_SUBPENDING_BIT:
	SUBSRCPND = 0x4a000018
	ldr	r1,=SUBSRCPND
	str	r0,[r1]
	mov pc,lr

HAL_CLEAR_EPENDING_BIT: .global HAL_CLEAR_EPENDING_BIT
HAL_CLEAR_EPENDING_BIT:
	EINTPEND = 0x560000a8
	ldr	r1,=EINTPEND	
	str	r0,[r1]
	mov pc,lr

HAL_MASK_INTR: .global HAL_MASK_INTR
HAL_MASK_INTR:
	mrs	r1, cpsr
	str	r1, [r0]
	orr	r1, r1, #NOINT
	msr	cpsr_cxsf, r1
	mov pc,lr


HAL_RESTORE_INTR: .global HAL_RESTORE_INTR
HAL_RESTORE_INTR:
	ldr	r1, [r0]
	msr	cpsr_cxsf, r1
	mov pc,lr


/*
==============
 CPSR I,F bit
==============
int HAL_SET_IF(void);
*/

HAL_SET_IF:  .global HAL_SET_IF
HAL_SET_IF:
   /* This function works only if the processor is in previliged mode.*/
   mrs r0,cpsr
   mov r1,r0
   orr r1,r1,#NOINT
   msr cpsr_cxsf,r1
   mov pc,lr

/*void HAL_WR_IF(int cpsrValue);*/
HAL_WR_IF:  .global HAL_WR_IF
HAL_WR_IF:
   /* This function works only if the processor is in previliged mode.*/
   msr cpsr_cxsf,r0
   MOV_PC_LR
/*
====================================
 MMU Cache/TLB/etc on/off functions
====================================
*/
R1_I	=	(1<<12)
R1_C	=	(1<<2)
R1_A	=	(1<<1)
R1_M    =	(1)
R1_iA	=	(1<<31)
R1_nF   =	(1<<30)


/* void MMU_EnableICache(void) */
HAL_MMU_ENABLE_ICACHE:  .global HAL_MMU_ENABLE_ICACHE
HAL_MMU_ENABLE_ICACHE:
   mrc p15,0,r0,c1,c0,0
   orr r0,r0,#R1_I
   mcr p15,0,r0,c1,c0,0
   MOV_PC_LR
   
HAL_MMU_DISABLE_ICACHE:  .global HAL_MMU_DISABLE_ICACHE
HAL_MMU_DISABLE_ICACHE:
   mrc p15,0,r0,c1,c0,0
   bic r0,r0,#R1_I
   mcr p15,0,r0,c1,c0,0
   MOV_PC_LR

HAL_MMU_INVALIDATE_ICACHE:  .global HAL_MMU_INVALIDATE_ICACHE
HAL_MMU_INVALIDATE_ICACHE:
   mcr p15,0,r0,c7,c5,0
   MOV_PC_LR
   
   
   
HAL_MMU_DISABLE_DCACHE:  .global HAL_MMU_DISABLE_DCACHE
HAL_MMU_DISABLE_DCACHE:
   mrc p15,0,r0,c1,c0,0
   bic r0,r0,#R1_C
   mcr p15,0,r0,c1,c0,0
   MOV_PC_LR

HAL_MMU_CLEAN_INVALIDATE_DCACHE_INDEX:  .global HAL_MMU_CLEAN_INVALIDATE_DCACHE_INDEX
HAL_MMU_CLEAN_INVALIDATE_DCACHE_INDEX:
   ;r0=index
   mcr p15,0,r0,c7,c10,2
   MOV_PC_LR

HAL_MMU_DISABLE:  .global HAL_MMU_DISABLE
HAL_MMU_DISABLE:
   mrc p15,0,r0,c1,c0,0
   bic r0,r0,#R1_M
   mcr p15,0,r0,c1,c0,0
   MOV_PC_LR

HAL_MMU_INVALIDATE_TLB:  .global HAL_MMU_INVALIDATE_TLB
HAL_MMU_INVALIDATE_TLB:
   mcr p15,0,r0,c8,c7,0
   MOV_PC_LR
   

HAL_MMU_SetTTBase:		 .global HAL_MMU_SetTTBase    
HAL_MMU_SetTTBase:
   mcr p15,0,r0,c2,c0,0
   MOV_PC_LR
   
HAL_MMU_SetDomain:		 .global HAL_MMU_SetDomain    
HAL_MMU_SetDomain:
   mcr p15,0,r0,c3,c0,0
   MOV_PC_LR
   
HAL_MMU_EnableICache:	 .global HAL_MMU_EnableICache    
HAL_MMU_EnableICache:
   mrc p15,0,r0,c1,c0,0
   orr r0,r0,#R1_I
   mcr p15,0,r0,c1,c0,0
   MOV_PC_LR

HAL_MMU_DisableICache:   .global HAL_MMU_DisableICache
HAL_MMU_DisableICache:
   mrc p15,0,r0,c1,c0,0
   bic r0,r0,#R1_I
   mcr p15,0,r0,c1,c0,0
   MOV_PC_LR


HAL_MMU_EnableDCache:    .global HAL_MMU_EnableDCache
HAL_MMU_EnableDCache:
   mrc p15,0,r0,c1,c0,0
   orr r0,r0,#R1_C
   mcr p15,0,r0,c1,c0,0
   MOV_PC_LR


HAL_MMU_DisableDCache:   .global HAL_MMU_DisableDCache
HAL_MMU_DisableDCache:
   mrc p15,0,r0,c1,c0,0
   bic r0,r0,#R1_C
   mcr p15,0,r0,c1,c0,0
   MOV_PC_LR

HAL_MMU_EnableAlignFault: .global HAL_MMU_EnableAlignFault
HAL_MMU_EnableAlignFault:
   mrc p15,0,r0,c1,c0,0
   orr r0,r0,#R1_A
   mcr p15,0,r0,c1,c0,0
   MOV_PC_LR

HAL_MMU_DisableAlignFault: .global HAL_MMU_DisableAlignFault
HAL_MMU_DisableAlignFault:
   mrc p15,0,r0,c1,c0,0
   bic r0,r0,#R1_A
   mcr p15,0,r0,c1,c0,0
   MOV_PC_LR


HAL_MMU_EnableMMU:         .global HAL_MMU_EnableMMU
HAL_MMU_EnableMMU:
   mrc p15,0,r0,c1,c0,0
   orr r0,r0,#R1_M
   mcr p15,0,r0,c1,c0,0
   MOV_PC_LR


HAL_MMU_DisableMMU:         .global HAL_MMU_DisableMMU
HAL_MMU_DisableMMU:
   mrc p15,0,r0,c1,c0,0
   bic r0,r0,#R1_M
   mcr p15,0,r0,c1,c0,0
   MOV_PC_LR
   
HAL_MMU_SetProcessId:       .global HAL_MMU_SetProcessId   
HAL_MMU_SetProcessId:
   ;r0= pid
   mcr p15,0,r0,c13,c0,0
   MOV_PC_LR

   




/*

;void CLR_IF(void);
   EXPORT  CLR_IF
CLR_IF
    ;This function works only if the processor is in previliged mode.
   mrs r0,cpsr
   bic r0,r0,#NOINT
   msr cpsr_cxsf,r0
   MOV_PC_LR

   	EXPORT	outportw
outportw	strh	r0, [r1]
	MOV_PC_LR

	EXPORT	inportw
inportw	ldrh	r0, [r0]
	MOV_PC_LR

	EXPORT	YOAV_SWI
YOAV_SWI
	stmfd	sp!,{lr}
	SWI 2
	ldmfd	sp!,{pc}



;====================================
; MMU Cache/TLB/etc on/off functions
;====================================
R1_I	EQU	(1<<12)
R1_C	EQU	(1<<2)
R1_A	EQU	(1<<1)
R1_M    EQU	(1)
R1_iA	EQU	(1<<31)
R1_nF   EQU	(1<<30)

;void MMU_EnableICache(void)
   EXPORT MMU_EnableICache
MMU_EnableICache
   mrc p15,0,r0,c1,c0,0
   orr r0,r0,#R1_I
   mcr p15,0,r0,c1,c0,0
   MOV_PC_LR

;void MMU_DisableICache(void)
   EXPORT MMU_DisableICache
MMU_DisableICache
   mrc p15,0,r0,c1,c0,0
   bic r0,r0,#R1_I
   mcr p15,0,r0,c1,c0,0
   MOV_PC_LR

;void MMU_EnableDCache(void)
   EXPORT MMU_EnableDCache
MMU_EnableDCache
   mrc p15,0,r0,c1,c0,0
   orr r0,r0,#R1_C
   mcr p15,0,r0,c1,c0,0
   MOV_PC_LR

;void MMU_DisableDCache(void)
   EXPORT MMU_DisableDCache
MMU_DisableDCache
   mrc p15,0,r0,c1,c0,0
   bic r0,r0,#R1_C
   mcr p15,0,r0,c1,c0,0
   MOV_PC_LR

;void MMU_EnableAlignFault(void)
   EXPORT MMU_EnableAlignFault
MMU_EnableAlignFault
   mrc p15,0,r0,c1,c0,0
   orr r0,r0,#R1_A
   mcr p15,0,r0,c1,c0,0
   MOV_PC_LR

;void MMU_DisableAlignFault(void)
   EXPORT MMU_DisableAlignFault
MMU_DisableAlignFault
   mrc p15,0,r0,c1,c0,0
   bic r0,r0,#R1_A
   mcr p15,0,r0,c1,c0,0
   MOV_PC_LR

;void MMU_EnableMMU(void)
   EXPORT MMU_EnableMMU
MMU_EnableMMU
   mrc p15,0,r0,c1,c0,0
   orr r0,r0,#R1_M
   mcr p15,0,r0,c1,c0,0
   MOV_PC_LR

;void MMU_DisableMMU(void)
   EXPORT MMU_DisableMMU
MMU_DisableMMU
   mrc p15,0,r0,c1,c0,0
   bic r0,r0,#R1_M
   mcr p15,0,r0,c1,c0,0
   MOV_PC_LR

;void MMU_SetFastBusMode(void)
; FCLK:HCLK= 1:1
  EXPORT MMU_SetFastBusMode
MMU_SetFastBusMode
   mrc p15,0,r0,c1,c0,0
   bic r0,r0,#R1_iA:OR:R1_nF
   mcr p15,0,r0,c1,c0,0
   MOV_PC_LR

;void MMU_SetAsyncBusMode(void)
; FCLK:HCLK= 1:2
   EXPORT MMU_SetAsyncBusMode
MMU_SetAsyncBusMode
   mrc p15,0,r0,c1,c0,0
   orr r0,r0,#R1_nF:OR:R1_iA
   mcr p15,0,r0,c1,c0,0
   MOV_PC_LR

;=========================
; Set TTBase
;=========================
;void MMU_SetTTBase(int base)
   EXPORT MMU_SetTTBase
MMU_SetTTBase
   ;ro=TTBase
   mcr p15,0,r0,c2,c0,0
   MOV_PC_LR

;=========================
; Set Domain
;=========================
;void MMU_SetDomain(int domain)
   EXPORT MMU_SetDomain
MMU_SetDomain
   ;ro=domain
   mcr p15,0,r0,c3,c0,0
   MOV_PC_LR

;=========================
; ICache/DCache functions
;=========================
;void MMU_InvalidateIDCache(void)
   EXPORT MMU_InvalidateIDCache
MMU_InvalidateIDCache
   mcr p15,0,r0,c7,c7,0
   MOV_PC_LR

;void MMU_InvalidateICache(void)
   EXPORT MMU_InvalidateICache
MMU_InvalidateICache
   mcr p15,0,r0,c7,c5,0
   MOV_PC_LR

;void MMU_InvalidateICacheMVA(U32 mva)
   EXPORT MMU_InvalidateICacheMVA
MMU_InvalidateICacheMVA
   ;r0=mva
   mcr p15,0,r0,c7,c5,1
   MOV_PC_LR

;void MMU_PrefetchICacheMVA(U32 mva)
   EXPORT MMU_PrefetchICacheMVA
MMU_PrefetchICacheMVA
   ;r0=mva
   mcr p15,0,r0,c7,c13,1
   MOV_PC_LR

;void MMU_InvalidateDCache(void)
   EXPORT MMU_InvalidateDCache
MMU_InvalidateDCache
   mcr p15,0,r0,c7,c6,0
   MOV_PC_LR

;void MMU_DrainWriteBuffer(U32 mva)
   EXPORT MMU_DrainWriteBuffer
MMU_DrainWriteBuffer
;   stmfd	sp!,{r0}
;   mov r0, #0
   mcr p15,0,r0,c7,c10,4
;   ldmfd	sp!,{r0}
   MOV_PC_LR

;void MMU_InvalidateDCacheMVA(U32 mva)
   EXPORT MMU_InvalidateDCacheMVA
MMU_InvalidateDCacheMVA
   ;r0=mva
   mcr p15,0,r0,c7,c6,1
   MOV_PC_LR

;void MMU_CleanDCacheMVA(U32 mva)
   EXPORT MMU_CleanDCacheMVA
MMU_CleanDCacheMVA
   ;r0=mva
   mcr p15,0,r0,c7,c10,1
   MOV_PC_LR

;void MMU_CleanInvalidateDCacheMVA(U32 mva)
   EXPORT MMU_CleanInvalidateDCacheMVA
MMU_CleanInvalidateDCacheMVA
   ;r0=mva
   mcr p15,0,r0,c7,c14,1
   MOV_PC_LR

;void MMU_CleanDCacheIndex(U32 index)
   EXPORT MMU_CleanDCacheIndex
MMU_CleanDCacheIndex
   ;r0=index
   mcr p15,0,r0,c7,c10,2
   MOV_PC_LR

;void MMU_CleanInvalidateDCacheIndex(U32 index)
   EXPORT MMU_CleanInvalidateDCacheIndex
MMU_CleanInvalidateDCacheIndex
   ;r0=index
   mcr p15,0,r0,c7,c14,2
   MOV_PC_LR

;void MMU_WaitForInterrupt(void)
   EXPORT MMU_WaitForInterrupt
MMU_WaitForInterrupt
   mcr p15,0,r0,c7,c0,4
   MOV_PC_LR

;===============
; TLB functions
;===============
;voic MMU_InvalidateTLB(void)
   EXPORT MMU_InvalidateTLB
MMU_InvalidateTLB
   mcr p15,0,r0,c8,c7,0
   MOV_PC_LR

;void MMU_InvalidateITLB(void)
   EXPORT MMU_InvalidateITLB
MMU_InvalidateITLB
   mcr p15,0,r0,c8,c5,0
   MOV_PC_LR

;void MMU_InvalidateITLBMVA(U32 mva)
   EXPORT MMU_InvalidateITLBMVA
MMU_InvalidateITLBMVA
   ;ro=mva
   mcr p15,0,r0,c8,c5,1
   MOV_PC_LR

;void MMU_InvalidateDTLB(void)
	EXPORT MMU_InvalidateDTLB
MMU_InvalidateDTLB
	mcr p15,0,r0,c8,c6,0
	MOV_PC_LR

;void MMU_InvalidateDTLBMVA(U32 mva)
	EXPORT MMU_InvalidateDTLBMVA
MMU_InvalidateDTLBMVA
	;r0=mva
	mcr p15,0,r0,c8,c6,1
	MOV_PC_LR

;=================
; Cache lock down
;=================
;void MMU_SetDCacheLockdownBase(U32 base)
   EXPORT MMU_SetDCacheLockdownBase
MMU_SetDCacheLockdownBase
   ;r0= victim & lockdown base
   mcr p15,0,r0,c9,c0,0
   MOV_PC_LR

;void MMU_SetICacheLockdownBase(U32 base)
   EXPORT MMU_SetICacheLockdownBase
MMU_SetICacheLockdownBase
   ;r0= victim & lockdown base
   mcr p15,0,r0,c9,c0,1
   MOV_PC_LR

;=================
; TLB lock down
;=================
;void MMU_SetDTLBLockdown(U32 baseVictim)
   EXPORT MMU_SetDTLBLockdown
MMU_SetDTLBLockdown
   ;r0= baseVictim
   mcr p15,0,r0,c10,c0,0
   MOV_PC_LR

;void MMU_SetITLBLockdown(U32 baseVictim)
   EXPORT MMU_SetITLBLockdown
MMU_SetITLBLockdown
   ;r0= baseVictim
   mcr p15,0,r0,c10,c0,1
   MOV_PC_LR

;============
; Process ID
;============
;void MMU_SetProcessId(U32 pid)
   EXPORT MMU_SetProcessId
MMU_SetProcessId
   ;r0= pid
   mcr p15,0,r0,c13,c0,0
   MOV_PC_LR

   END
*/

.ltorg
DataTable:
/*
; Memory configuration should be optimized for best performance
; The following parameter is not optimized.
; Memory access cycle parameter strategy
; 1) The memory settings is  safe parameters even at HCLK=75Mhz.
; 2) SDRAM refresh period is for HCLK<=75Mhz.
*/

	.word 0x56000000
	.word 0x00000001
	.word 0x00000002
	.word 0x00000003
	.word 0x00000004
	.word 0x00000005
	.word 0x00000006
	.word 0x00000007
	.word 0x00000008

	.word 0x56000020
	.word 0x00000000
	.word 0x00000000
	.word 0x00000000
	.word 0x00000000
	.word 0x00000000
	.word 0x00000000
	.word 0x00000000
	.word 0x00000000

	.word 0x0

.end
